unit ZipLst16;

interface

uses Classes;

function GetZipContent(ZipName: String; Liste: TStrings): LongInt;
         {Ergebnis: Anzahl der gefundenen Zip-Eintrge}
implementation
uses WinTypes, SysUtils, Dialogs;

type ZipDirEntry = packed Record
  Version          : WORD;
  Flag             : WORD;
  CompressionMethod: WORD;
  DateTime         : Longint;
  CRC32            : Longint;
  CompressedSize   : Longint;
  UncompressedSize : Longint;
  FileNameLength   : WORD;
  ExtraFieldLength : WORD;
end;

const
  LocalFileHeaderSign   = $04034b50; { 'PK03' }
  CentralFileHeaderSign = $02014b50; { 'PK12' }
  EndCentralDirSign     = $06054b50; { 'PK56' }

function GetZipContent(ZipName: String; Liste: TStrings): LongInt;
var nZ, nRes, Sign: LongInt;
    cName: array[0..255]of Char;
    sName: String;
    ZipStream: TFileStream;
    ZipDir: ZipDirEntry;
begin
 nZ:=0;                                  {Zahl der gefundenen ZIP-Eintrge}
 Result:=0;
 if not FileExists(ZipName) then begin
  ShowMessage('Datei "'+ZipName+'" nicht vorhanden!'); Exit;
 end;
 ZipStream:=TFileStream.Create(ZipName,fmOpenRead);
 try
  repeat
   nRes:=ZipStream.Read(Sign,SizeOf(Sign));
   if nRes<>SizeOf(Sign) then                       {Ende ohne Zip-Kennung}
    raise EStreamError.Create('Fehler 1 beim Lesen des Zip-Files');
   if Sign=LocalFileHeaderSign then begin            {Zip-Kennung gefunden}
    if (ZipStream.Read(ZipDir.Version,SizeOf(ZipDir))=SizeOf(ZipDir)) and
       (ZipStream.Read(cName,ZipDir.FileNameLength)=ZipDir.FileNameLength)
    then cName[ZipDir.FileNameLength]:=#0            {#0 an Namen anhngen}
    else raise EStreamError.Create('Fehler 2 beim Lesen des Zip-Files');
    if ZipStream.Position+ZipDir.ExtraFieldLength+ZipDir.CompressedSize>ZipStream.Size-22 then
     raise EStreamError.Create('Fehler 3 beim Lesen des Zip-Files');
    if ZipDir.ExtraFieldLength>0 then begin
     nRes:=ZipStream.Position+ZipDir.ExtraFieldLength;
     if ZipStream.Seek(ZipDir.ExtraFieldLength,soFromCurrent)<>nRes then
      raise EStreamError.Create('Fehler 4 beim Lesen des Zip-Files');
    end;
    nRes:=ZipStream.Position+ZipDir.CompressedSize;
    if ZipStream.Seek(ZipDir.CompressedSize,soFromCurrent)<>nRes then
     raise EStreamError.Create('Fehler 5 beim Lesen des Zip-Files');
    sName:=FormatDateTime('dd.mm.yy hh:nn ',FileDateToDateTime(ZipDir.DateTime))+
           FormatFloat('000000# ',ZipDir.UnCompressedSize)+StrPas(cName);
    Liste.Add(sName);
    Inc(nZ);                                      {Zhler fr Zip-Eintrge}
   end;                                    {Signum war LocalFileHeaderSign}
  until (Sign=CentralFileHeaderSign) or (Sign=EndCentralDirSign);
 finally
  ZipStream.Free;
  Result:=nZ;
 end; {try}
end;

end.
